library angular2.src.core.compiler.dynamic_component_loader;

import "package:angular2/di.dart"
    show Key, Injector, Injectable, ResolvedBinding, Binding, bind;
import "compiler.dart" show Compiler;
import "package:angular2/src/facade/lang.dart"
    show Type, BaseException, stringify, isPresent;
import "package:angular2/src/facade/async.dart" show Future;
import "package:angular2/src/core/compiler/view_manager.dart"
    show AppViewManager;
import "element_injector.dart" show ElementRef;
import "view.dart" show AppView; /**
 * @exportedAs angular2/view
 */

class ComponentRef {
  ElementRef location;
  dynamic instance;
  AppView componentView;
  Function _dispose;
  ComponentRef(ElementRef location, dynamic instance, AppView componentView,
      Function dispose) {
    this.location = location;
    this.instance = instance;
    this.componentView = componentView;
    this._dispose = dispose;
  }
  get injector {
    return this.location.injector;
  }
  get hostView {
    return this.location.hostView;
  }
  dispose() {
    this._dispose();
  }
} /**
 * Service for dynamically loading a Component into an arbitrary position in the internal Angular
 * application tree.
 *
 * @exportedAs angular2/view
 */
@Injectable()
class DynamicComponentLoader {
  Compiler _compiler;
  AppViewManager _viewManager;
  DynamicComponentLoader(Compiler compiler, AppViewManager viewManager) {
    this._compiler = compiler;
    this._viewManager = viewManager;
  } /**
   * Loads a component into the location given by the provided ElementRef. The loaded component
   * receives injection as if it in the place of the provided ElementRef.
   */
  Future<ComponentRef> loadIntoExistingLocation(
      typeOrBinding, ElementRef location, [Injector injector = null]) {
    var binding = this._getBinding(typeOrBinding);
    return this._compiler.compile(binding.token).then((componentProtoView) {
      var componentView = this._viewManager.createDynamicComponentView(
          location, componentProtoView, binding, injector);
      var dispose = () {
        throw new BaseException("Not implemented");
      };
      return new ComponentRef(location,
          location.elementInjector.getDynamicallyLoadedComponent(),
          componentView, dispose);
    });
  } /**
   * Loads a component in the element specified by elementOrSelector. The loaded component receives
   * injection normally as a hosted view.
   */
  Future<ComponentRef> loadIntoNewLocation(typeOrBinding,
      ElementRef parentComponentLocation, dynamic elementOrSelector,
      [Injector injector = null]) {
    return this._compiler
        .compileInHost(this._getBinding(typeOrBinding))
        .then((hostProtoView) {
      var hostView = this._viewManager.createInPlaceHostView(
          parentComponentLocation, elementOrSelector, hostProtoView, injector);
      var newLocation = hostView.elementInjectors[0].getElementRef();
      var component = hostView.elementInjectors[0].getComponent();
      var dispose = () {
        this._viewManager.destroyInPlaceHostView(
            parentComponentLocation, hostView);
      };
      return new ComponentRef(
          newLocation, component, hostView.componentChildViews[0], dispose);
    });
  } /**
   * Loads a component next to the provided ElementRef. The loaded component receives
   * injection normally as a hosted view.
   */
  Future<ComponentRef> loadNextToExistingLocation(
      typeOrBinding, ElementRef location, [Injector injector = null]) {
    var binding = this._getBinding(typeOrBinding);
    return this._compiler.compileInHost(binding).then((hostProtoView) {
      var hostView = location.viewContainer.create(-1, hostProtoView, injector);
      var newLocation = hostView.elementInjectors[0].getElementRef();
      var component = hostView.elementInjectors[0].getComponent();
      var dispose = () {
        var index = location.viewContainer.indexOf(hostView);
        location.viewContainer.remove(index);
      };
      return new ComponentRef(
          newLocation, component, hostView.componentChildViews[0], dispose);
    });
  }
  _getBinding(typeOrBinding) {
    var binding;
    if (typeOrBinding is Binding) {
      binding = typeOrBinding;
    } else {
      binding = bind(typeOrBinding).toClass(typeOrBinding);
    }
    return binding;
  }
}
