library angular2.src.core.compiler.view;

import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, Map, StringMapWrapper, List;
import "package:angular2/change_detection.dart"
    show
        AST,
        Locals,
        ChangeDispatcher,
        ProtoChangeDetector,
        ChangeDetector,
        ChangeRecord,
        BindingRecord,
        DirectiveRecord,
        DirectiveIndex,
        ChangeDetectorRef;
import "element_injector.dart"
    show
        ProtoElementInjector,
        ElementInjector,
        PreBuiltObjects,
        DirectiveBinding;
import "element_binder.dart" show ElementBinder;
import "package:angular2/src/reflection/types.dart" show SetterFn;
import "package:angular2/src/facade/lang.dart"
    show IMPLEMENTS, int, isPresent, isBlank, BaseException;
import "package:angular2/src/render/api.dart" as renderApi;

class AppViewContainer {
  List<AppView> views;
  AppViewContainer() {
    // The order in this list matches the DOM order.
    this.views = [];
  }
} /**
 * Const of making objects: http://jsperf.com/instantiate-size-of-object
 *
 */
@IMPLEMENTS(ChangeDispatcher)
class AppView implements ChangeDispatcher {
  renderApi.ViewRef render; /// This list matches the _nodes list. It is sparse, since only Elements have ElementInjector
  List<ElementInjector> rootElementInjectors;
  List<ElementInjector> elementInjectors;
  ChangeDetector changeDetector;
  List<AppView> componentChildViews; /// Host views that were added by an imperative view.
  /// This is a dynamically growing / shrinking array.
  List<AppView> imperativeHostViews;
  List<AppViewContainer> viewContainers;
  List<PreBuiltObjects> preBuiltObjects;
  AppProtoView proto;
  renderApi.Renderer renderer; /**
   * The context against which data-binding expressions in this view are evaluated against.
   * This is always a component instance.
   */
  dynamic context; /**
   * Variables, local to this view, that can be used in binding expressions (in addition to the
   * context). This is used for thing like `<video #player>` or
   * `<li template="for #item of items">`, where "player" and "item" are locals, respectively.
   */
  Locals locals;
  AppView(renderApi.Renderer renderer, AppProtoView proto, Map protoLocals) {
    this.render = null;
    this.proto = proto;
    this.changeDetector = null;
    this.elementInjectors = null;
    this.rootElementInjectors = null;
    this.componentChildViews = null;
    this.viewContainers =
        ListWrapper.createFixedSize(this.proto.elementBinders.length);
    this.preBuiltObjects = null;
    this.context = null;
    this.locals = new Locals(null, MapWrapper.clone(protoLocals));
    this.renderer = renderer;
    this.imperativeHostViews = [];
  }
  init(ChangeDetector changeDetector, List elementInjectors,
      List rootElementInjectors, List preBuiltObjects,
      List componentChildViews) {
    this.changeDetector = changeDetector;
    this.elementInjectors = elementInjectors;
    this.rootElementInjectors = rootElementInjectors;
    this.preBuiltObjects = preBuiltObjects;
    this.componentChildViews = componentChildViews;
  }
  void setLocal(String contextName, value) {
    if (!this.hydrated()) throw new BaseException(
        "Cannot set locals on dehydrated view.");
    if (!MapWrapper.contains(this.proto.variableBindings, contextName)) {
      return;
    }
    var templateName = MapWrapper.get(this.proto.variableBindings, contextName);
    this.locals.set(templateName, value);
  }
  bool hydrated() {
    return isPresent(this.context);
  } /**
   * Triggers the event handlers for the element and the directives.
   *
   * This method is intended to be called from directive EventEmitters.
   *
   * @param {string} eventName
   * @param {*} eventObj
   * @param {int} binderIndex
   */
  void triggerEventHandlers(String eventName, eventObj, int binderIndex) {
    var locals = MapWrapper.create();
    MapWrapper.set(locals, "\$event", eventObj);
    this.dispatchEvent(binderIndex, eventName, locals);
  } // dispatch to element injector or text nodes based on context
  void notifyOnBinding(BindingRecord b, dynamic currentValue) {
    if (b.isElement()) {
      this.renderer.setElementProperty(
          this.render, b.elementIndex, b.propertyName, currentValue);
    } else {
      // we know it refers to _textNodes.
      this.renderer.setText(this.render, b.elementIndex, currentValue);
    }
  }
  getDirectiveFor(DirectiveIndex directive) {
    var elementInjector = this.elementInjectors[directive.elementIndex];
    return elementInjector.getDirectiveAtIndex(directive.directiveIndex);
  }
  getDetectorFor(DirectiveIndex directive) {
    var childView = this.componentChildViews[directive.elementIndex];
    return isPresent(childView) ? childView.changeDetector : null;
  } // implementation of EventDispatcher#dispatchEvent
  // returns false if preventDefault must be applied to the DOM event
  bool dispatchEvent(
      num elementIndex, String eventName, Map<String, dynamic> locals) {
    // Most of the time the event will be fired only when the view is in the live document.
    // However, in a rare circumstance the view might get dehydrated, in between the event
    // queuing up and firing.
    var allowDefaultBehavior = true;
    if (this.hydrated()) {
      var elBinder = this.proto.elementBinders[elementIndex];
      if (isBlank(elBinder.hostListeners)) return allowDefaultBehavior;
      var eventMap = elBinder.hostListeners[eventName];
      if (isBlank(eventMap)) return allowDefaultBehavior;
      MapWrapper.forEach(eventMap, (expr, directiveIndex) {
        var context;
        if (identical(directiveIndex, -1)) {
          context = this.context;
        } else {
          context = this.elementInjectors[elementIndex]
              .getDirectiveAtIndex(directiveIndex);
        }
        var result = expr.eval(context, new Locals(this.locals, locals));
        if (isPresent(result)) {
          allowDefaultBehavior = allowDefaultBehavior && result;
        }
      });
    }
    return allowDefaultBehavior;
  }
} /**
 *
 */
class AppProtoView {
  List<ElementBinder> elementBinders;
  ProtoChangeDetector protoChangeDetector;
  Map variableBindings;
  Map protoLocals;
  int textNodesWithBindingCount;
  List bindings;
  AppProtoView parentProtoView;
  List _variableBindings;
  Map _directiveRecordsMap;
  List _directiveRecords;
  renderApi.ProtoViewRef render;
  AppProtoView(
      renderApi.ProtoViewRef render, ProtoChangeDetector protoChangeDetector) {
    this.render = render;
    this.elementBinders = [];
    this.variableBindings = MapWrapper.create();
    this.protoLocals = MapWrapper.create();
    this.protoChangeDetector = protoChangeDetector;
    this.parentProtoView = null;
    this.textNodesWithBindingCount = 0;
    this.bindings = [];
    this._directiveRecordsMap = MapWrapper.create();
    this._variableBindings = null;
    this._directiveRecords = null;
  } //TODO: Tobias or Victor. Moving it into the constructor.
  // this work should be done the constructor of AppProtoView once we separate
  // AppProtoView and ProtoViewBuilder
  List getVariableBindings() {
    if (isPresent(this._variableBindings)) {
      return this._variableBindings;
    }
    this._variableBindings = isPresent(this.parentProtoView)
        ? ListWrapper.clone(this.parentProtoView.getVariableBindings())
        : [];
    MapWrapper.forEach(this.protoLocals, (v, local) {
      ListWrapper.push(this._variableBindings, local);
    });
    return this._variableBindings;
  } //TODO: Tobias or Victor. Moving it into the constructor.
  // this work should be done the constructor of ProtoView once we separate
  // AppProtoView and ProtoViewBuilder
  List getdirectiveRecords() {
    if (isPresent(this._directiveRecords)) {
      return this._directiveRecords;
    }
    this._directiveRecords = [];
    for (var injectorIndex = 0;
        injectorIndex < this.elementBinders.length;
        ++injectorIndex) {
      var pei = this.elementBinders[injectorIndex].protoElementInjector;
      if (isPresent(pei)) {
        for (var directiveIndex = 0;
            directiveIndex < pei.numberOfDirectives;
            ++directiveIndex) {
          ListWrapper.push(this._directiveRecords,
              this._getDirectiveRecord(injectorIndex, directiveIndex));
        }
      }
    }
    return this._directiveRecords;
  }
  void bindVariable(String contextName, String templateName) {
    MapWrapper.set(this.variableBindings, contextName, templateName);
    MapWrapper.set(this.protoLocals, templateName, null);
  }
  ElementBinder bindElement(ElementBinder parent, int distanceToParent,
      ProtoElementInjector protoElementInjector,
      [DirectiveBinding componentDirective = null,
      DirectiveBinding viewportDirective = null]) {
    var elBinder = new ElementBinder(this.elementBinders.length, parent,
        distanceToParent, protoElementInjector, componentDirective,
        viewportDirective);
    ListWrapper.push(this.elementBinders, elBinder);
    return elBinder;
  } /**
   * Adds a text node binding for the last created ElementBinder via bindElement
   */
  void bindTextNode(AST expression) {
    var textNodeIndex = this.textNodesWithBindingCount++;
    var b = BindingRecord.createForTextNode(expression, textNodeIndex);
    ListWrapper.push(this.bindings, b);
  } /**
   * Adds an element property binding for the last created ElementBinder via bindElement
   */
  void bindElementProperty(AST expression, String setterName) {
    var elementIndex = this.elementBinders.length - 1;
    var b =
        BindingRecord.createForElement(expression, elementIndex, setterName);
    ListWrapper.push(this.bindings, b);
  } /**
   * Adds an host property binding for the last created ElementBinder via bindElement
   */
  void bindHostElementProperty(
      AST expression, String setterName, DirectiveIndex directiveIndex) {
    var b = BindingRecord.createForHostProperty(
        directiveIndex, expression, setterName);
    ListWrapper.push(this.bindings, b);
  } /**
   * Adds an event binding for the last created ElementBinder via bindElement.
   *
   * If the directive index is a positive integer, the event is evaluated in the context of
   * the given directive.
   *
   * If the directive index is -1, the event is evaluated in the context of the enclosing view.
   *
   * @param {string} eventName
   * @param {AST} expression
   * @param {int} directiveIndex The directive index in the binder or -1 when the event is not bound
   *                             to a directive
   */
  void bindEvent(List<renderApi.EventBinding> eventBindings,
      [int directiveIndex = -1]) {
    var elBinder = this.elementBinders[this.elementBinders.length - 1];
    var events = elBinder.hostListeners;
    if (isBlank(events)) {
      events = StringMapWrapper.create();
      elBinder.hostListeners = events;
    }
    for (var i = 0; i < eventBindings.length; i++) {
      var eventBinding = eventBindings[i];
      var eventName = eventBinding.fullName;
      var event = StringMapWrapper.get(events, eventName);
      if (isBlank(event)) {
        event = MapWrapper.create();
        StringMapWrapper.set(events, eventName, event);
      }
      MapWrapper.set(event, directiveIndex, eventBinding.source);
    }
  } /**
   * Adds a directive property binding for the last created ElementBinder via bindElement
   */
  void bindDirectiveProperty(
      num directiveIndex, AST expression, String setterName, SetterFn setter) {
    var elementIndex = this.elementBinders.length - 1;
    var directiveRecord =
        this._getDirectiveRecord(elementIndex, directiveIndex);
    var b = BindingRecord.createForDirective(
        expression, setterName, setter, directiveRecord);
    ListWrapper.push(this.bindings, b);
  }
  DirectiveRecord _getDirectiveRecord(
      num elementInjectorIndex, num directiveIndex) {
    var id = elementInjectorIndex * 100 + directiveIndex;
    var protoElementInjector = this.elementBinders[
        elementInjectorIndex].protoElementInjector;
    if (!MapWrapper.contains(this._directiveRecordsMap, id)) {
      var binding =
          protoElementInjector.getDirectiveBindingAtIndex(directiveIndex);
      var changeDetection = binding.changeDetection;
      MapWrapper.set(this._directiveRecordsMap, id, new DirectiveRecord(
          new DirectiveIndex(elementInjectorIndex, directiveIndex),
          binding.callOnAllChangesDone, binding.callOnChange, changeDetection));
    }
    return MapWrapper.get(this._directiveRecordsMap, id);
  }
}
