library angular2.src.core.compiler.view_container_ref;

import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, List;
import "package:angular2/di.dart" show Injector;
import "package:angular2/src/core/compiler/element_injector.dart" as eiModule;
import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "view.dart" as viewModule;
import "view_manager.dart" as avmModule; /**
 * @exportedAs angular2/view
 */

class ViewContainerRef {
  avmModule.AppViewManager _viewManager;
  eiModule.ElementRef _location;
  viewModule.AppProtoView _defaultProtoView;
  ViewContainerRef(avmModule.AppViewManager viewManager,
      eiModule.ElementRef location, viewModule.AppProtoView defaultProtoView) {
    this._viewManager = viewManager;
    this._location = location;
    this._defaultProtoView = defaultProtoView;
  }
  _getViews() {
    var vc = this._location.hostView.viewContainers[
        this._location.boundElementIndex];
    return isPresent(vc) ? vc.views : [];
  }
  void clear() {
    for (var i = this.length - 1; i >= 0; i--) {
      this.remove(i);
    }
  }
  viewModule.AppView get(num index) {
    return this._getViews()[index];
  }
  get length {
    return this._getViews().length;
  } // TODO(rado): profile and decide whether bounds checks should be added
  // to the methods below.
  viewModule.AppView create([num atIndex = -1,
      viewModule.AppProtoView protoView = null, Injector injector = null]) {
    if (atIndex == -1) atIndex = this.length;
    if (isBlank(protoView)) {
      protoView = this._defaultProtoView;
    }
    return this._viewManager.createViewInContainer(
        this._location, atIndex, protoView, injector);
  }
  viewModule.AppView insert(viewModule.AppView view, [num atIndex = -1]) {
    if (atIndex == -1) atIndex = this.length;
    return this._viewManager.attachViewInContainer(
        this._location, atIndex, view);
  }
  indexOf(viewModule.AppView view) {
    return ListWrapper.indexOf(this._getViews(), view);
  }
  void remove([num atIndex = -1]) {
    if (atIndex == -1) atIndex = this.length - 1;
    this._viewManager.destroyViewInContainer(this._location, atIndex);
  } /**
   * The method can be used together with insert to implement a view move, i.e.
   * moving the dom nodes while the directives in the view stay intact.
   */
  viewModule.AppView detach([num atIndex = -1]) {
    if (atIndex == -1) atIndex = this.length - 1;
    return this._viewManager.detachViewInContainer(this._location, atIndex);
  }
}
