library angular2.src.render.dom.compiler.compiler;

import "package:angular2/di.dart" show Injectable;
import "package:angular2/src/facade/async.dart" show PromiseWrapper, Future;
import "package:angular2/src/facade/lang.dart" show BaseException;
import "../../api.dart" show ViewDefinition, ProtoViewDto;
import "compile_pipeline.dart" show CompilePipeline;
import "package:angular2/src/render/dom/compiler/template_loader.dart"
    show TemplateLoader;
import "compile_step_factory.dart" show CompileStepFactory, DefaultStepFactory;
import "package:angular2/change_detection.dart" show Parser;
import "../shadow_dom/shadow_dom_strategy.dart"
    show
        ShadowDomStrategy; /**
 * The compiler loads and translates the html templates of components into
 * nested ProtoViews. To decompose its functionality it uses
 * the CompilePipeline and the CompileSteps.
 */

class Compiler {
  TemplateLoader _templateLoader;
  CompileStepFactory _stepFactory;
  Compiler(CompileStepFactory stepFactory, TemplateLoader templateLoader) {
    this._templateLoader = templateLoader;
    this._stepFactory = stepFactory;
  }
  Future<ProtoViewDto> compile(ViewDefinition template) {
    var tplPromise = this._templateLoader.load(template);
    return PromiseWrapper.then(tplPromise,
        (el) => this._compileTemplate(template, el), (_) {
      throw new BaseException(
          '''Failed to load the template "${ template . componentId}"''');
    });
  }
  Future<ProtoViewDto> _compileTemplate(ViewDefinition template, tplElement) {
    var subTaskPromises = [];
    var pipeline = new CompilePipeline(
        this._stepFactory.createSteps(template, subTaskPromises));
    var compileElements;
    compileElements = pipeline.process(tplElement, template.componentId);
    var protoView = compileElements[0].inheritedProtoView.build();
    if (subTaskPromises.length > 0) {
      return PromiseWrapper.all(subTaskPromises).then((_) => protoView);
    } else {
      return PromiseWrapper.resolve(protoView);
    }
  }
}
@Injectable()
class DefaultCompiler extends Compiler {
  DefaultCompiler(Parser parser, ShadowDomStrategy shadowDomStrategy,
      TemplateLoader templateLoader)
      : super(
          new DefaultStepFactory(parser, shadowDomStrategy), templateLoader) {
    /* super call moved to initializer */;
  }
}
